#!/bin/bash
# fakeroot
# http://ftp.de.debian.org/debian/pool/main/f/fakeroot/fakeroot_1.14.4.orig.tar.bz2
# http://packages.debian.org/changelogs/pool/main/f/fakeroot/current/changelog
# http://packages.debian.org/fakeroot
#
# script:  til (rob.til.freedman~AT~gmail.com)
set -e -u #-x

version=1.14.4
pkgname=fakeroot-$version

arch=${arch:-i486}
bld_ver=64.1
package=$pkgname-$arch-$bld_ver

cpu=${cpu:-i686}
CFLAGS="-O2 -march=$arch -mtune=$cpu" 

CWD=$(pwd)
TMP=${TMP:-$CWD}
SRC=$TMP/$pkgname
PKG=$TMP/$package

# prepare the package build directory
[ -d $PKG ] && rm -fr $PKG
mkdir -p $PKG/{install,usr/{doc,src}/$pkgname}

[ -d $SRC ] && rm -fr $SRC
mkdir -p $SRC/{obj-sysv,obj-tcp}

# fakeroot can be built to either use SYSV IPC or TCP for message passing.
# Cases for which the SYSV IPC version fails or causes problems but for which 
# fakeroot-tcp has been observed to work well include the items listed below.
#
# * Multithreaded applications (using pthread)
# * Running under realtime-preempt kernel

do_make() {
    parm=${1:-} 
    CFLAGS="$CFLAGS" ../configure --prefix=/usr --mandir=/usr/man \
            --libdir=/usr/lib/libfakeroot ${parm}  || return 1
    make || return 1
    make check || return 1
    make DESTDIR=$PKG install
    rm  $PKG/usr/lib/libfakeroot/libfakeroot.{so,a,la}
    return 0
}

tar xf $CWD/fakeroot_${version}.orig.tar.bz2 -C $TMP
cd $SRC
#autoreconf -fi
./bootstrap
# method of IPC to use: tcp
cd $SRC/obj-tcp
do_make "--program-suffix=-tcp --with-ipc=tcp" || exit 1

# rename lib 
mv $PKG/usr/lib/libfakeroot/libfakeroot-0.so \
    $PKG/usr/lib/libfakeroot/libfakeroot-tcp.so    

# method of IPC to use: sysv (default) 
cd $SRC/obj-sysv
do_make || exit 1

# rename lib 
mv $PKG/usr/lib/libfakeroot/libfakeroot-0.so \
    $PKG/usr/lib/libfakeroot/libfakeroot.so

# helperline
(cd $PKG;find . -printf "%Y %M %u:%g %p %l\n") >$CWD/files

# docs
cd $SRC
cp -a AUTHORS BUGS COPYING DEBUG doc/README.saving \
  $PKG/usr/doc/$pkgname

cd $PKG
# strip symbols
find . | xargs file | grep -e "executable" -e "shared object" \
  | grep ELF | cut -f 1 -d : | xargs -r strip --strip-unneeded 

# compress man pages
find usr/man -iname '*-tcp.1' -delete
find usr/man -iname '*.1' | xargs gzip -9
# 
for mdir in `find usr/man -type d -iname 'man1'`; do
    ( cd $mdir;
      ln -s faked.1.gz faked-tcp.1.gz
      ln -s fakeroot.1.gz fakeroot-tcp.1.gz
    )
done
#
cat <<EOF > usr/doc/$pkgname/known_issues
Known issues

The fakeroot environment is imposed upon a process by using the C-library's LD_PRELOAD
environment variable. libfakeroot is preloaded by the dynamic linker whenever it loads
a binary.  This means that fakeroot does not work with statically linked binaries.

There is also another side-effect. Since libfakeroot is loaded into the same process
image with the "victim" program, they share the same file descriptor table.
Some programs (such as the configure scripts) use hard-coded descriptor numbers.
libfakeroot needs one file descriptor for its communication socket, and if the program
starts to use the same file descriptor, there will be trouble.

fakeroot tries to monitor the status of its descriptor so that it can open a new socket
if the descriptor has been changed. If you start seeing messages about hijacked file
descriptors, you can try to make fakeroot use some other file descriptor with the
  --fd-base option.
Its default value is (descriptor_table_size - 100).

# http://alioth.debian.org/projects/fakeroot/
# http://fakeroot.alioth.debian.org/
EOF

#       |-----handy-ruler------------------------------------------------------|
cat <<EOF > install/slack-desc 
fakeroot: fakeroot ( provides a faked root environment )
fakeroot:
fakeroot: This is done by setting LD_PRELOAD to libfakeroot.so and wrapper
fakeroot: around getuid, chown, chmod, mknod, stat ...
fakeroot:
fakeroot: Cases for which the SYSV IPC version fakeroot causes problems,
fakeroot: fakeroot-tcp has been observed to work well for the following 
fakeroot: items:
fakeroot: * Multithreaded applications (using pthread)
fakeroot: * Running under realtime-preempt kernel
EOF

# build script
cp -a $CWD/build-fakeroot.sh usr/src/$pkgname

# done
fakeroot /sbin/makepkg -l y -c n $CWD/$package.txz

cd $CWD
PKG_SRC="http://zenwalk.pinguix.com/user-accounts/tilderb/extra/ap/fakeroot/build-fakeroot.sh
http://ftp.debian.org/debian/pool/main/f/fakeroot/fakeroot_${version}.tar.gz"

echo "$PKG_SRC" \
    > $package.src
md5sum $package.txz \
    > $package.md5
:>$package.dep
